#!/usr/bin/env python

"""bcs.find.py

    Functions to simplify finding BCS files.
"""

import logging

import sys
import os

from datetime import datetime, date, time, timedelta
from dateutil import relativedelta as rel_date
import pytz

from numpy import array
import numpy as np
import glob
import pandas as pd


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTIONS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def replace_subpath(file_path, subpath_replace_dict):
    """Replace specified parts of file path according to supplied dictionary.

        file_path: Fully qualified file path (dir + file).
        subpath_replace_dict: {old_subpath: new_subpath, ...}.
            *) All replacements performed, in order of items().
            *) Suggested to use collections.OrderedDict when order important.

        RETURNS: Fully qualified file path (dir + file) after all replacements.
    """

    new_file_path = file_path

    for (old_subpath, new_subpath) in subpath_replace_dict.items():

        new_file_path = new_file_path.replace(old_subpath, new_subpath)

        logging.debug(
                "Scan file (updated path): {:s} <WAS: {:s}, NOW: {:s}>".format(
                new_file_path, old_subpath, new_subpath,
                )
            )

    logging.debug("Scan file (updated path): {:s}".format(new_file_path))

    return(new_file_path)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def daterange(start_date, end_date, endpoint=True):
    """Iterable generator for range of dates. RETURNS datetime.date"""
    if endpoint:
        last_day = 1
    else:
        last_day = 0
    for ndays in range(int ((end_date - start_date).days + last_day)):
        yield start_date + timedelta(days=ndays)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def find_data_files_in_date_range(
        data_scan_numbers = None,
        first_date = None,
        last_date = None,
        data_file_base = "TrajScan",
        data_path_base = "",
        ):
    """Find all BCS data files matching date rage and name criteria

        *) Search for all BCS data files created: first_date to last_date
           ...which contain data_file_base in the filename

        data_scan_numbers: list or array of scan numbers for data files
            *) If None specified, return all files matching data_file_base
        first_date: datetime.date object for path name of earliest data files
            *) Defaults to the past week
        last_date: datetime.date object for path name of most recent data files
            *) Defaults to today
        data_file_base: Base name for data files
            *) e.g., "TrajScan", "SigScan", "TimeScan", "InstrumentScan"
        data_path_base: Base path for data file search

        RETURNS: list of matching filenames
    """

    if data_scan_numbers is None:
        data_file_search_strings = array([data_file_base])
    else:
        data_scan_numbers = array(data_scan_numbers)
        data_file_search_strings = array([
            "{}{:4d}".format(data_file_base, int(data_scan_number))
            for data_scan_number in data_scan_numbers
            ])

    if first_date is None:
        first_date = date.today() - timedelta(days=7)
        # first_date = date.today() - rel_date.relativedelta((days=7)

    if last_date is None:
        last_date = date.today()

    matching_file_paths = list()

    for data_file_date in daterange(first_date, last_date, endpoint=True):
        for data_file_search_string in data_file_search_strings:
            data_subpath_base = "{}/{}/".format(
                data_path_base,
                data_file_date.strftime("%Y"),
                )
            if not os.path.exists(data_subpath_base):
                data_subpath_base = data_path_base
            data_path_search_string = "{}/{}/{}*".format(
                data_subpath_base,
                data_file_date.strftime("%y%m%d"),
                data_file_search_string,
                )
            logging.debug("Search: " + data_path_search_string)
            match_results = glob.glob(data_path_search_string)
            if len(match_results) > 0:
                matching_file_paths += match_results

    logging.info("\n\nfind_data_files_in_date_range...")
    logging.info("len(matching_file_paths): {}".format(len(matching_file_paths)))

    return(matching_file_paths)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def main():
    """The main routine."""

    print(__doc__ + "\n")

    return(0)
    

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if __name__ == '__main__':
    main()


#!/usr/bin/env python

"""bcs.data.py

    Functions to inspect the BCS data files that will be ingested.
"""

import logging

import sys
import os

from datetime import datetime, date, time, timedelta
from dateutil import relativedelta as rel_date
import pytz

from numpy import array
import numpy as np
import glob
import pandas as pd

from .scans import (
    get_scan_file_path, replace_subpath, get_scan_header_line_number,
    get_scan_line_numbers, import_scan_file,
    )


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTIONS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_data_file_numbers(data_file_path):
    """Extract the (scan, file, repeat) numbers from the data file path.

        data_file_path: Fully qualified file path (dir + file) of data.

        RETURNS: (scan, file, repeat) numbers found in the data file path
            *) XxxxScan$$$$-RRRR_FFFF.txt --> (scan, file, repeat) = ($, F, R)
    """

    (scan_number, file_number, repeat_number) = (None, None, None)

    file_basename = os.path.basename(data_file_path)
    file_basename_parts = os.path.splitext(file_basename)

    file_basename_parts = file_basename_parts[0].rsplit('_', 1)
    if len(file_basename_parts) > 1:
        file_number = int(file_basename_parts[1])

    file_basename_parts = file_basename_parts[0].rsplit('-', 1)
    if len(file_basename_parts) > 1:
        repeat_number = int(file_basename_parts[1])

    file_basename_parts = file_basename_parts[0].rsplit('Scan', 1)
    if len(file_basename_parts) > 1:
        try:
            scan_number = int(file_basename_parts[1])
        except ValueError as e:
            # This is a new Beamline/Integrated scan
            file_basename_parts = file_basename_parts[1].rsplit(' ', 1)
            scan_number = int(file_basename_parts[0])
            if len(file_basename_parts) > 1:
                file_number = int(file_basename_parts[-1])
    
    return(scan_number, file_number, repeat_number)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def read_data_file(
        data_file_path,
        subpath_replace_dict=None,
        **kwargs
        ):

    """Loads the data from all files into a PANDAS DataFrame.

        data_file_path: full path of data file
        subpath_replace_dict: {old_subpath: new_subpath, ...}
            *) Used for locating input scan file

        RETURNS: PANDAS DataFrame containing data from imported file
    """

    if subpath_replace_dict is None:
        subpath_replace_dict = dict()

    logging.debug("\n\nRead_data_file...")

    logging.debug("Opening: {}".format(data_file_path))

    isTimeScan = False
    file_name = os.path.basename(data_file_path)
    if file_name.startswith("Time"):
        isTimeScan = True

    with open(data_file_path, 'r') as data_file:
        for (header_row, file_line) in enumerate(data_file):
            logging.debug("[{}]: {}".format(header_row, file_line))
            if file_line.startswith("Time"):
                break
            if file_line[0].isdigit() and not isTimeScan:
                header_row -= 1
                break

    data = pd.read_table(
        data_file_path,
        delimiter='\t',
        header=header_row,
        skip_blank_lines=False,
        )

    data["filename"] = os.path.splitext(
        os.path.basename(data_file_path)
    )[0]

    if len(data) > 0:
        logging.debug("...filename: {}".format(data["filename"][0]))
    # logging.debug("...filename: {}".format(data["filename"].values[0]))

    df = data

    return df

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def main():
    """The main routine."""

    print(__doc__ + "\n")

    return(0)
    

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if __name__ == '__main__':
    main()


#!/usr/bin/env python

"""bcs.scans.py

    Functions to inspect the BCS scan files that generated the data.
"""

import logging

import sys
import os

from datetime import datetime, date, time, timedelta
from dateutil import relativedelta as rel_date
import pytz

from numpy import array, mod
import numpy as np
import glob
import pandas as pd

from .find import find_data_files_in_date_range, replace_subpath


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTIONS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_scan_file_path(data_file_path):
    """Extract the scan file from the data file header.

        data_file_path: Fully qualified file path (dir + file) of data.

        RETURNS: Fully qualified file path (dir + file) of scan input.
            "" = Not found
    """

    scan_file_used = False

    with open(data_file_path, 'r') as data_file:
        for (header_linenum, file_line) in enumerate(data_file):

            logging.debug(header_linenum, file_line)

            if file_line.startswith("From File"):
                scan_file_used = True
                continue
            if scan_file_used and file_line.rstrip():
                scan_file_path_raw = file_line.rstrip()
                scan_file_path = scan_file_path_raw.replace("\\", "/")
                if scan_file_path == "All Magnets":
                    # This is a quirk of a new file format
                    continue
                logging.debug("Scan file: {:s}".format(scan_file_path))
                return(scan_file_path)
                break
            if file_line[0].isdigit():
                # This data file did not use an input scan file
                return("")

    return("")

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_scan_header_line_number(scan_file_path):
    """Extract line number of motor headers from scan file.

        scan_file_path: Fully qualified path (dir + file) of scan file.

        RETURNS: Zero-based line number of motor header row.
            -1 = Not found
    """

    with open(scan_file_path, 'r') as scan_file:
        for (header_linenum, file_line) in enumerate(scan_file):

            logging.debug(header_linenum, file_line)

            # if file_line[0].isdigit() or file_line.lower().startswith("file"):
            if (
                    file_line[0].isdigit() or 
                    file_line.lower().startswith("file") or
                    (file_line[0]=='-' and file_line[1].isdigit())
                    ):
                header_linenum -= 1
                return(header_linenum)

    return(-1)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_scan_line_numbers(scan_file_path, file_number=1):
    """Extract line numbers corresponding to file_number from scan file.

        scan_file_path: Fully qualified path (dir + file) of scan file.
        file_number: One-based file number (extracted from data file path).

        RETURNS: Tuple of zero-based line numbers (first, last) of motor
                    positions for output file_number specified.
            first = -1, if file_number not found
    """

    (first_line, last_line) = (-1, -1)  # Not found

    header_linenum = get_scan_header_line_number(scan_file_path)
    if header_linenum == -1:
        return(first_line, last_line)

    file_number = file_number or 1  # If file_number is None
    output_file_number = 1
    if output_file_number == file_number:
        next_linenum = header_linenum + 1
        (first_line, last_line) = (next_linenum, next_linenum)

    with open(scan_file_path, 'r') as scan_file:
        for (linenum, file_line) in enumerate(scan_file):

            # Move past header line
            if linenum <= header_linenum:
                continue

            logging.debug(linenum, file_line)

            if file_line.lower().startswith("file"):
                output_file_number += 1
                if output_file_number == file_number:
                    next_linenum = linenum + 1
                    (first_line, last_line) = (next_linenum, next_linenum)
                elif output_file_number > file_number:
                    return (first_line, last_line)
            else:
                if output_file_number == file_number:
                    last_line = linenum
        else:
            # This is a new Beamline/Integrated scan
            max_file_number = output_file_number
            if file_number > max_file_number:
                file_number = mod(file_number, max_file_number)
                if file_number == 0:
                    file_number = max_file_number
                (first_line, 
                 last_line,
                 ) = get_scan_line_numbers(scan_file_path, file_number)
    return (first_line, last_line)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def import_scan_file(scan_file_path, file_number=1):
    """Import motor positions from scan file into PANDAS DataFrame.

        scan_file_path: Fully qualified path (dir + file) of scan file.
        file_number: One-based file number (extracted from data file path).

        RETURNS: PANDAS DataFrame of imported motor positions
                    for output file_number specified.
    """

    header_linenum = get_scan_header_line_number(scan_file_path)
    (first_linenum, last_linenum) = get_scan_line_numbers(
        scan_file_path, file_number)

    skiprows = np.arange(header_linenum + 1, first_linenum)
    logging.debug("skiprows: {}".format(skiprows))

    nrows = 1 + last_linenum - first_linenum
    logging.debug("nrows: {}".format(nrows))

    try:
        df = pd.read_table(
            scan_file_path,
            delimiter='\t',
            header=header_linenum,
            skip_blank_lines=False,
            skiprows=skiprows,
            nrows=nrows,
        )
    except pd.errors.ParserError:
        # First line is a comment, and header line does not end with '\t'
        df = pd.read_table(
            scan_file_path,
            delimiter='\t',
            header=header_linenum,
            skip_blank_lines=False,
            skiprows=skiprows + [first_linenum],
            nrows=(nrows - 1),
        )
    # Ignore comment lines in the scan file
    def is_comment(value: str):
        return value.strip().startswith('#')
    comment_rows = df.iloc[:, 0].astype(str).apply(is_comment)
    comment_indices = df[comment_rows].index
    df.drop(comment_indices, inplace=True)
    
    # Check whether this is a Flying Scan; extract flying motor name
    if header_linenum > 0:
        with open(scan_file_path, 'r') as scan_file:
            first_line = scan_file.readline().rstrip()
            logging.debug(f"first_line: {first_line}")
        
        # first_line_parts = first_line.lower().rsplit('flying ', 1)
        first_line_parts = first_line.replace(
            'flying ', 'Flying ').rsplit('Flying ', 1)
        logging.debug(f"first_line_parts #1: {first_line_parts}")
        if len(first_line_parts) > 1:
            first_line = first_line_parts[-1]
            first_line_parts = first_line.rsplit('(', 1)
            logging.debug(f"first_line_parts #2: {first_line_parts}")
            if len(first_line_parts) > 1:
                first_line = first_line_parts[0]
                flying_name = first_line.strip()
                logging.debug(f"flying_name: {flying_name}")
                
                # Add Flying Motor to column names
                if df.columns[-1].startswith("Unnamed"):
                    df.columns = [*df.columns[:-1], flying_name]
                else:
                    # Repair scan file with correct number of columns
                    index_name = df.columns[0]
                    col_names = [*df.columns[1:], flying_name]
                    df.columns = col_names
                    df.index.rename(index_name, inplace=True)
                    df = df.reset_index()
                print(f'{df.columns=}')

    return(df)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def is_flying_scan(scan_file_path, file_number=1):
    """Check whether a flying scan generated the output file_number specified

        scan_file_path: Fully qualified path (dir + file) of scan file.
        file_number: One-based file number (extracted from data file path).

        RETURNS: True if a flying motor is found; False otherwise
    """

    header_linenum = get_scan_header_line_number(scan_file_path)
    (first_linenum, last_linenum) = get_scan_line_numbers(
        scan_file_path, file_number)

    skiprows = np.arange(header_linenum + 1, first_linenum)
    logging.debug("skiprows: {}".format(skiprows))

    nrows = 1 + last_linenum - first_linenum
    logging.debug("nrows: {}".format(nrows))

    df = pd.read_table(
        scan_file_path,
        delimiter='\t',
        header=header_linenum,
        skip_blank_lines=False,
        skiprows=skiprows,
        nrows=nrows,
    )
    
    # Check whether this is a Flying Scan; extract flying motor name
    if header_linenum > 0:
        with open(scan_file_path, 'r') as scan_file:
            first_line = scan_file.readline().rstrip()
            logging.debug(f"first_line: {first_line}")
        
        # first_line_parts = first_line.lower().rsplit('flying ', 1)
        first_line_parts = first_line.replace(
            'flying ', 'Flying ').rsplit('Flying ', 1)
        logging.debug(f"first_line_parts #1: {first_line_parts}")
        if len(first_line_parts) > 1:
            first_line = first_line_parts[-1]
            first_line_parts = first_line.rsplit('(', 1)
            logging.debug(f"first_line_parts #2: {first_line_parts}")
            if len(first_line_parts) > 1:
                first_line = first_line_parts[0]
                flying_name = first_line.strip()
                logging.debug(f"flying_name: {flying_name}")

                return True
                
                # Return name of flying motor also?
                # return(True, flying_name)

    return False

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def main():
    """The main routine."""

    print(__doc__ + "\n")

    return(0)
    

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if __name__ == '__main__':
    main()


#!/usr/bin/env python

"""bcs.ingest.py

    Extract data and metadata from BCS data files.
"""

import logging

logger = logging.getLogger()
DEBUG = False
if DEBUG:
    logger.setLevel(logging.DEBUG)
else:
    logger.setLevel(logging.INFO)

import sys
import os
import argparse

from datetime import datetime, date, time, timedelta
from dateutil import relativedelta as rel_date
import pytz

from numpy import array, source
import numpy as np
import glob
import pandas as pd
from pandas.errors import ParserError as pandas_ParserError
import itertools as it
from collections import OrderedDict

from .data import get_data_file_numbers, read_data_file
from .find import find_data_files_in_date_range, replace_subpath, daterange
from .scans import get_scan_file_path, import_scan_file, is_flying_scan

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Configure environment
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
default_tz_name = 'America/Los_Angeles'
default_timezone = pytz.timezone(default_tz_name)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# FUNCTIONS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_file_timestamps(file_path: str):
    """Return file timestamps as dict"""
    file_stats = os.stat(file_path)
    sys_platform = sys.platform
    if DEBUG:
        logging.debug(f"{sys_platform = }")
    if sys_platform.lower() == "windows":
        file_create_timestamp = file_stats.st_ctime	# Windows
    elif sys_platform.lower() == "darwin":
        file_create_timestamp = file_stats.st_birthtime	# Mac
    else:
        # We're probably on Linux. No easy way to get creation dates here,
        # so we'll settle for when its content was last modified.
        file_create_timestamp = file_stats.st_mtime
    return dict(
        file_path=file_path,
        created_timestamp=file_create_timestamp,
        modified_timestamp=os.path.getmtime(file_path),
        )

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_filename(file_path: str):
    """Extract file name from file path"""
    return os.path.basename(file_path)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_filename_base(file_path: str):
    """Extract file name base (no extension) from file path"""
    filename = get_filename(file_path)
    filename_base = filename.rsplit('.', 1)[0]
    return filename_base

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def timestamp_to_string(timestamp: float, tz_name=default_tz_name):
    """Convert timestamp to timezone-aware string"""
    def format_datetime(datetime_obj):
        # return datetime.strftime(datetime_obj, "%Y-%m-%d %H:%M:%S.%f [%z]")
        return datetime.strftime(datetime_obj, "%Y-%m-%d %H:%M:%S [%z]")
    # Add TZ awareness
    timezone = pytz.timezone(tz_name)
    datetime_obj = datetime.fromtimestamp(timestamp, tz=timezone)
    return format_datetime(datetime_obj)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def get_data_file_header(
        data_file_path: str, 
        subpath_replace_dict: dict=None,
        ):
    """Extract the data file header.

        data_file_path: Fully qualified file path (dir + file) of data.
        subpath_replace_dict: {old_subpath: new_subpath, ...}
            *) Used for locating input scan file

        RETURNS: dict with header fields
    """

    if subpath_replace_dict is None:
        subpath_replace_dict = dict()

    if DEBUG:
        logging.debug(f"***{data_file_path}")
    
    header_info = dict(
        scan_type=None,
        date=None,
        motors=[], 
        motor_first=[],
        motor_last=[],
        motor_step=[],
        # motor_velocity=[],  # Must be undefined for non-flying
        flying=False,
        )
    get_motor_name = False
    get_pause_motor_name = False
    get_scan_file_path = False
    get_memo = False

    with open(data_file_path, 'r') as data_file:
        for (header_linenum, file_line) in enumerate(data_file):

            if DEBUG:
                logging.debug(header_linenum, file_line)

            if get_motor_name:
                motor_name = file_line.strip()
                header_info["motors"].append(motor_name)
                get_motor_name = False
                continue
            if get_pause_motor_name:
                motor_name = file_line.strip()
                header_info["pause_motor"] = motor_name
                get_pause_motor_name = False
                get_scan_file_path = True
                continue
            if get_scan_file_path:
                scan_file_path_raw = file_line.rstrip()
                # Make the next transformation optional?
                scan_file_path = scan_file_path_raw.replace("\\", "/")
                header_info["scan_file_path"] = scan_file_path
                get_scan_file_path = False
                # Get scan file info
                (
                 scan_number, 
                 file_number, 
                 repeat_number,
                 ) = get_data_file_numbers(data_file_path)
                scan_file_path = replace_subpath(
                    scan_file_path, subpath_replace_dict)
                # logging.debug(f"{get_filename_base(data_file_path)}")
                # logging.debug(f"...{(scan_number, file_number, repeat_number) = }")
                motor_df = import_scan_file(scan_file_path, file_number)
                # logging.debug(f"...{motor_df = }")
                header_info["motors"] = motor_df.columns.values
                header_info["motor_values"] = motor_df.values.T
                if is_flying_scan(scan_file_path, file_number):
                    header_info["flying"] = True
                    # Move flying motor values to primary motor values
                    flying_motor_values = header_info["motor_values"][-1]
                    key_names = [
                        "motor_first",
                        "motor_last",
                        "motor_step",
                        "motor_velocity",
                        ]
                    for key_name in key_names:
                        header_info[key_name] = []
                    for motor_value_str in flying_motor_values:
                        motor_value_str = motor_value_str.strip().lower().lstrip('flying')
                        motor_value_str = motor_value_str.strip().lstrip('(').rstrip(')')
                        for (motor_value, key_name) in zip(
                                motor_value_str.split(','),
                                key_names,
                                ):
                            header_info[key_name].append(float(motor_value))
                    header_info["motor_values"] = header_info["motor_values"][:-1]
                continue
            if get_memo:
                value_str = file_line.rstrip()
                header_info["memo"] = value_str
                get_memo = False
                continue
            if file_line.startswith("Date: "):
                date_str = file_line.strip().split("Date: ", 1)[1]
                data_date = datetime.strptime(date_str, "%m/%d/%Y").date()
                header_info["date"] = data_date
                continue
            if file_line.startswith("Flying Scan"):
                header_info["flying"] = True
                header_info["scan_type"] = "Single Motor"
                get_motor_name = True
                continue
            if file_line.startswith("Start, Stop, Increment"):
                header_info["scan_type"] = "Single Motor"
                get_motor_name = True
                continue
            if file_line.startswith("From File"):
                # scan_file_used = True
                header_info["scan_type"] = "Trajectory"
                get_pause_motor_name = True
                continue
            # TODO: Add Time Scan
            # TODO: Add Two Motor Scan
            # TODO: Add Image Scan
            if file_line.startswith("Start"):
                value_str = file_line.strip().split("Start: ", 1)[1]
                header_info["motor_first"] = [float(value_str)]
                continue
            if file_line.startswith("Stop"):
                value_str = file_line.strip().split("Stop: ", 1)[1]
                header_info["motor_last"] = [float(value_str)]
                continue
            if file_line.startswith("Increment"):
                value_str = file_line.strip().split("Increment: ", 1)[1]
                header_info["motor_step"] = [float(value_str)]
                continue
            if file_line.startswith("X Center"):
                value_str = file_line.strip().split("X Center: ", 1)[1]
                header_info["motor_velocity"] = [float(value_str)]
                continue
            if file_line.startswith("Delay "):
                value_str = file_line.strip().split("Delay After Move (s): ", 1)[1]
                header_info["delay_sec"] = float(value_str)
                continue
            if file_line.startswith("Count "):
                value_str = file_line.strip().split("Count Time (s): ", 1)[1]
                header_info["count_sec"] = float(value_str)
                continue
            if file_line.startswith("Scan Number"):
                value_str = file_line.strip().split("Scan Number: ", 1)[1]
                header_info["repeat_number"] = int(value_str)
                continue
            if file_line.startswith("Bi-directional"):
                value_str = file_line.strip().split("Bi-directional: ", 1)[1]
                header_info["bidirect"] = True if (
                    value_str=="Yes") else False
                continue
            if file_line.startswith("Stay "):
                value_str = file_line.strip().split("Stay at End: ", 1)[1]
                header_info["stay_at_end"] = bool(value_str)
                continue
            if file_line.startswith("Decription "):
                value_str = file_line.strip().split("Description Length: ", 1)[1]
                get_memo = bool(value_str)
                # Is length needed for multi-line memo?
                continue
            if file_line[0].isdigit():
                # We've gone past the header
                header_linenum -= 1
                header_info["motor_header_linenum"] = header_linenum
                break

    return(header_info)

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
def main():
    """The main routine."""
    return(0)
    

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if __name__ == '__main__':
    main()


#!/usr/bin/env python

"""version_id.py

   Prints PEP440-compliant version identifier to standard output
"""

__author__ = "Padraic Shafer"
__copyright__ = "Copyright (c) 2022, Padraic Shafer"
__credits__ = [__author__, ]
__license__ = ""
__maintainer__ = "Padraic Shafer"
__email__ = "PShafer@lbl.gov"
__status__ = "Development"

from als.bcs import __version__, __date__

import argparse


def main():
    parser = argparse.ArgumentParser(
        # usage=__doc__,
        description=__doc__,
        # formatter_class=SmartFormatter,
        )
    parser.add_argument(
        "--version",
        action="version",
        # version="{} {}".format(__package__, __version__),
        version=__version__,
        help="Display PEP440 version identifier",
        )
    args = parser.parse_args()

    # Default: Print version identifier
    if args == argparse.Namespace():
        parser.parse_args(["--version"])


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# This single line is required for "pkgutil-style namespace package"
# DO NOT EDIT this file
#######################
__path__ = __import__('pkgutil').extend_path(__path__, __name__)


#!/usr/bin/env python

__version__ = None  # This will be assigned later; see below
__date__ = None  # This will be assigned later; see below
__credits__ = None  # This will be assigned later; see below

try:

    from als.bcs._version import git_pieces_from_vcs as _git_pieces_from_vcs
    from als.bcs._version import run_command, register_vcs_handler
    from als.bcs._version import render as _render
    from als.bcs._version import render_pep440_auto
    from als.bcs._version import render_pep440_micro, render_pep440_develop
    from als.bcs._version import get_versions as _get_versions
    from als.bcs._version import get_config, get_keywords
    from als.bcs._version import git_versions_from_keywords
    from als.bcs._version import versions_from_parentdir
    from als.bcs._version import NotThisMethod


except ImportError:

    # Assumption is that _version.py was generated by 'versioneer.py'
    # for tarball distribution, which contains only static JSON version data

    from als.bcs._version import get_versions

    # from als.bcs._version import get_versions as _get_versions
    #
    # def get_versions():
    #     """Get version information or return default if unable to do so.
    #
    #     Extension to ._version.get_versions()
    #
    #     Additional functionality:
    #     Returns list of authors found in `git`
    #     """
    #     default_keys_values = {
    #         "version": "0+unknown",
    #         "full-revisionid": None,
    #         "dirty": None,
    #         "error": "unable to compute version",
    #         "date": None,
    #         "authors": [],
    #     }
    #
    #     return_key_values = _get_versions()
    #     return_key_values = dict(
    #         default_keys_values.items() + return_key_values.items()
    #         )
    #     return return_key_values


else:

    import os
    import sys

    def unique(seq):
        """Find unique elements in a list, without sorting

        Implementation does not depend on `numpy`
        * borrowed from https://stackoverflow.com/a/480227
        * credit to Markus Jarderot
        + https://www.peterbe.com/plog/uniqifiers-benchmark
        """
        seen = set()
        seen_add = seen.add
        return [x for x in seq if not (x in seen or seen_add(x))]

    @register_vcs_handler("git", "pieces_from_vcs")
    def git_pieces_from_vcs(
            tag_prefix, root, verbose, run_command=run_command):
        """Get version information from 'git' in the root of the source tree.

        Extension to ._version.git_pieces_from_vcs()

        Additional functionality:
        Extracts all commit authors, sorts unique authors chronologically,
        then adds them to `pieces["authors"]`, where `pieces` is the object
        that was returned by ._version.git_pieces_from_vcs()
        """
        pieces = _git_pieces_from_vcs(
            tag_prefix, root, verbose, run_command=run_command)

        GITS = ["git"]
        if sys.platform == "win32":
            GITS = ["git.cmd", "git.exe"]

        ##################################################
        # Added to retrieve list of authors
        (authors_raw, rc) = run_command(
            GITS, ["log", "--pretty=%an"], cwd=root)
        authors = [author.strip() for author in authors_raw.split('\n')]
        authors_unique = unique(authors)
        pieces["authors"] = list(reversed(authors_unique))

        return pieces


    def render(pieces, style):
        """Render the given version pieces into the requested style."""
        if pieces["error"]:
            return {"version": "unknown",
                    "full-revisionid": pieces.get("long"),
                    "dirty": None,
                    "error": pieces["error"],
                    "date": None,
                    "authors": None,
                    }

        if not style or style == "default":
            style = "pep440-auto"  # the default

        if style == "pep440-micro":
            rendered = render_pep440_micro(pieces)
        elif style == "pep440-develop":
            rendered = render_pep440_develop(pieces)
        elif style == "pep440-auto":
            rendered = render_pep440_auto(pieces)
        else:
            return_key_values = _render(pieces, style)
            return_key_values["authors"] = pieces["authors"]
            return return_key_values

        return {"version": rendered, "full-revisionid": pieces["long"],
                "dirty": pieces["dirty"], "error": None,
                "date": pieces.get("date"), "authors": pieces["authors"]}


    def get_versions():
        """Get version information or return default if unable to do so.

        Extension to ._version.get_versions()

        Additional functionality:
        Returns list of authors found in `git`
        """
        # I am in _version.py, which lives at ROOT/VERSIONFILE_SOURCE.
        # If we have __file__, we can work backwards from there to the root.
        # Some py2exe/bbfreeze/non-CPython implementations don't do __file__,
        # in which case we can only use expanded keywords.

        cfg = get_config()
        verbose = cfg.verbose

        default_keys_values = {
            "version": "0+unknown",
            "full-revisionid": None,
            "dirty": None,
            "error": "unable to compute version",
            "date": None,
            "authors": [],
        }

        try:
            return git_versions_from_keywords(get_keywords(), cfg.tag_prefix,
                                              verbose)
        except NotThisMethod:
            pass

        try:
            root = os.path.realpath(__file__)
            # versionfile_source is the relative path from the top of the
            # source tree (where the .git directory might live) to this file.
            # Invert this to find the root from __file__.
            for i in cfg.versionfile_source.split('/'):
                root = os.path.dirname(root)
        except NameError:
            return default_keys_values

        try:
            pieces = git_pieces_from_vcs(cfg.tag_prefix, root, verbose)
            return render(pieces, cfg.style)
        except NotThisMethod:
            pass

        try:
            if cfg.parentdir_prefix:
                return versions_from_parentdir(
                    cfg.parentdir_prefix, root, verbose)
        except NotThisMethod:
            pass

        return_key_values = _get_versions()
        return_key_values = dict(
            list( default_keys_values.items() )
            + list( return_key_values.items() )
            )
        return return_key_values


__version__ = get_versions()["version"]
__date__ = get_versions()["date"]
__credits__ = get_versions()["authors"]

del get_versions



#!/usr/bin/env python

"""A setuptools-based setup module for als.bcs
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

# Non-qualified import of `versioneer`; local file is intended
import versioneer

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Get the dependencies from external file
with open(path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    install_requires = f.read().splitlines()
    install_requires = [
        dep.strip() for dep in install_requires 
            if not dep.strip().startswith('-')
    ]

setup(
    name='als.bcs',

    version=versioneer.get_version(), 
    cmdclass=versioneer.get_cmdclass(), 

    description='Extract data and metadata from text data files'
                ' that were generated by BCS at the ALS', 
    long_description=long_description, 
    long_description_content_type='text/markdown',

    url='https://github.com/als-computing/als.bcs', 

    author='Padraic Shafer', 
    author_email='pshafer@lbl.gov', 

    classifiers=[ 
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers', 
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Information Analysis', 
        'Topic :: Utilities', 

        # Pick your license as you wish
        # 'License :: OSI Approved :: BSD License :: BSD-3-Clause-LBNL',
        'License :: OSI Approved :: BSD License', # 'BSD-3-Clause-LBNL'

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',

        'Operating System :: OS Independent',
        'Environment :: Console',
    ],

    keywords='ALS BCS Data Metadata', 

    packages=find_packages(exclude=['contrib', 'docs', 'tests']), 

    install_requires=install_requires, 

    extras_require={  # Optional
        # 'dev': ['check-manifest'],
        # 'test': ['coverage'],
    },

    include_package_data=True,

    entry_points={  # Optional
        'console_scripts': [
        ],
    },
)


#!/usr/bin/env python

__author__ = "Padraic Shafer"
__copyright__ = "Copyright (c) 2022, Padraic Shafer"
__credits__ = [__author__, ]
__license__ = ""
__maintainer__ = "Padraic Shafer"
__email__ = "PShafer@lbl.gov"
__status__ = "Development"

from .version import __version__, __date__, __credits__
